@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.11 2006/06/12 23:26:12 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/in-memory.lisp,v 1.13 2006/09/15 07:21:41 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -193,6 +193,50 @@ is reached."
         do (vector-push-extend (elt sequence index) vector))
   sequence)
 
+(defmethod stream-file-position ((stream vector-input-stream))
+  "Simply returns the index into the underlying vector."
+  (vector-stream-index stream))
+
+(defmethod (setf stream-file-position) (position-spec (stream vector-input-stream))
+  "Sets the index into the underlying vector if POSITION-SPEC is acceptable."
+  (setf (vector-stream-index stream)
+        (case position-spec
+          (:start 0)
+          (:end (vector-stream-end stream))
+          (otherwise
+           (unless (integerp position-spec)
+             (error "Unknown file position designator: ~S." position-spec))
+           (unless (<= 0 position-spec (vector-stream-end stream))
+             (error "File position designator ~S is out of bounds." position-spec))
+           position-spec)))
+  position-spec)
+
+(defmethod stream-file-position ((stream vector-output-stream))
+  "Simply returns the fill pointer of the underlying vector."
+  (fill-pointer (vector-stream-vector stream)))
+
+(defmethod (setf stream-file-position) (position-spec (stream vector-output-stream))
+  "Sets the fill pointer underlying vector if POSITION-SPEC is
+acceptable.  Adjusts the vector if necessary."
+  (let* ((vector (vector-stream-vector stream))
+         (total-size (array-total-size vector))
+         (new-fill-pointer
+          (case position-spec
+            (:start 0)
+            (:end
+             (warn "File position designator :END doesn't really make sense for an output stream.")
+             total-size)
+            (otherwise
+             (unless (integerp position-spec)
+               (error "Unknown file position designator: ~S." position-spec))
+             (unless (<= 0 position-spec array-total-size-limit)
+               (error "File position designator ~S is out of bounds." position-spec))
+             position-spec))))
+    (when (> new-fill-pointer total-size)
+      (adjust-array vector new-fill-pointer))
+    (setf (fill-pointer vector) new-fill-pointer)
+    position-spec))
+
 (defmethod make-in-memory-input-stream ((vector vector) &optional (start 0)
                                                                   (end (length vector)))
   "Returns a binary input stream which will supply, in order, the
