@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: FLEXI-STREAMS; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/flexi-streams/stream.lisp,v 1.22 2006/06/29 13:34:00 edi Exp $
+;;; $Header: /usr/local/cvsrep/flexi-streams/stream.lisp,v 1.26 2006/09/15 07:21:41 edi Exp $
 
 ;;; Copyright (c) 2005-2006, Dr. Edmund Weitz.  All rights reserved.
 
@@ -43,8 +43,7 @@ by this stream.  Can be changed on the fly.")
    (element-type :initform #+:lispworks 'lw:simple-char #-:lispworks 'character
                  :initarg :element-type
                  :accessor flexi-stream-element-type
-                 :documentation "The element type of this stream.
-Must be a subtype of CHARACTER."))
+                 :documentation "The element type of this stream."))
   (:documentation "A FLEXI-STREAM object is a stream that's
 `layered' atop an existing binary/bivalent stream in order to
 allow for multi-octet external formats.  FLEXI-STREAM itself is a
@@ -64,8 +63,7 @@ capabilities."))
   ((element-type :initarg :element-type
                  :reader flexi-stream-element-type-error-element-type))
   (:report (lambda (condition stream)
-             (format stream "Element type ~S should have been a ~
-subtype of CHARACTER."
+             (format stream "Element type ~S not allowed."
                      (flexi-stream-element-type-error-element-type condition))))
   (:documentation "Errors of this type are signaled if the flexi
 stream has a wrong element type."))
@@ -98,7 +96,8 @@ reasonable values."
   (declare (ignore initargs))
   (with-slots (external-format element-type)
       flexi-stream
-    (unless (subtypep element-type 'character)
+    (unless (or (subtypep element-type 'character)
+                (subtypep element-type 'octet))
       (error 'flexi-stream-element-type-error
              :element-type element-type
              :stream flexi-stream))
@@ -111,7 +110,8 @@ necessary."
 
 (defmethod (setf flexi-stream-element-type) :before (new-value (flexi-stream flexi-stream))
   "Checks whether the new value makes sense before it is set."
-  (unless (subtypep new-value 'character)
+  (unless (or (subtypep new-value 'character)
+              (subtypep new-value 'octet))
     (error 'flexi-stream-element-type-error
            :element-type new-value
            :stream flexi-stream)))
@@ -134,6 +134,15 @@ stream."
   "A flexi stream is open if its underlying stream is open."
   (open-stream-p (flexi-stream-stream stream)))
 
+(defmethod stream-file-position ((stream flexi-stream))
+  "Dispatch to method for underlying stream."
+  (stream-file-position (flexi-stream-stream stream)))
+
+(defmethod (setf stream-file-position) (position-spec (stream flexi-stream))
+  "Dispatch to method for underlying stream."
+  (setf (stream-file-position (flexi-stream-stream stream))
+        position-spec))
+
 (defclass flexi-output-stream (flexi-stream fundamental-binary-output-stream
                                             fundamental-character-output-stream)
   ((column :initform 0
@@ -185,20 +194,19 @@ MAKE-FLEXI-STREAM instead."))
 (defun make-flexi-stream (stream &rest args
                                  &key (external-format (make-external-format :iso-8859-1))
                                       element-type column position)
-  "Creates and returns a new flexi stream.  STREAM must be an
-open binary or `bivalent' stream, i.e. it must be capable of
-reading/writing octets with READ-SEQUENCE and/or WRITE-SEQUENCE.
-The resulting flexi stream is an input stream if and only if
-STREAM is an input stream.  Likewise, it's an output stream if
-and only if STREAM is an output stream.  ELEMENT-TYPE must be a
-subtype of CHARACTER, the default is LW:SIMPLE-CHAR.
-EXTERNAL-FORMAT must be an EXTERNAL-FORMAT object or a symbol or
-a list denoting such an object.  COLUMN is the initial column of
-the stream which is either a non-negative integer or NIL.  The
-COLUMN argument is only used for output streams.  POSITION \(only
-used for input streams) should be an integer and it denotes the
-position the stream is in - it will be increased by one for each
-octet read."
+  "Creates and returns a new flexi stream.  STREAM must be an open
+binary or `bivalent' stream, i.e. it must be capable of
+reading/writing octets with READ-SEQUENCE and/or WRITE-SEQUENCE.  The
+resulting flexi stream is an input stream if and only if STREAM is an
+input stream.  Likewise, it's an output stream if and only if STREAM
+is an output stream.  The default for ELEMENT-TYPE is LW:SIMPLE-CHAR
+on LispWorks and CHARACTER on other Lisps.  EXTERNAL-FORMAT must be an
+EXTERNAL-FORMAT object or a symbol or a list denoting such an object.
+COLUMN is the initial column of the stream which is either a
+non-negative integer or NIL.  The COLUMN argument is only used for
+output streams.  POSITION \(only used for input streams) should be an
+integer and it denotes the position the stream is in - it will be
+increased by one for each octet read."
   ;; these arguments are ignored - they are only there to provide a
   ;; meaningful parameter list for IDEs
   (declare (ignore element-type column position))
